/*
 * Cache Manager
 * 
 * V2.0.7
 */

/* global WebOSManager, SOAPClient, XPathResult, utils, base64js, LibraryManager, LayoutManager, ScheduleManager, ERRORS, XMLDocument, application, logic, Logger */

"use strict";

/**
 * @classdesc Provide cache functionality.
 * @type CacheManager
 * @version 2.0.7
 * @since 2.0.7
 */
var CacheManager = new function () {
  // <editor-fold defaultstate="collapsed" desc="Private variables">
  var v_LayoutDurations = {};
  var cycleSubList = [];
  /** Holds the layouts that are in a campaign ...*/
  var campaignOrder = {};
  /** Holds the campaig that played before ...*/
  var currentGroupKey = -1;
  /**
   * Keep track of how many plays a layout did per hour.
   * @type {Object} Object with layout ids as keys and plays per hour and last (re)set time.
   */
  var maxPlaysPerHour = {};
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Private functions">
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Public functions">
  
  /**
   * Set the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} duration The duration of the layout in seconds.
   * @returns {undefined} 
   */
  this.setLayoutDuration = function(id , duration){
    v_LayoutDurations[id] = duration;
  };
  
  /**
   * Get the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} defaultDuration The duration of the layout in seconds if not previously set.
   * @returns {String|Number}  The duration of the layout in seconds.
   */
  this.getLayoutDuration = function(id , defaultDuration){
    if (Object.keys(v_LayoutDurations).indexOf(id)!==-1){
        return v_LayoutDurations[id];
    }
    return defaultDuration;
  };
  this.setCycleListSequence  = function (media) {
    var j;
    var pcycleref ;
    var mediaflag = 0;// 1 if media exits !

    // if (Object.keys(cycleSubList).indexOf(media.parentWidgetId)===-1){
    if (!cycleSubList[media.parentWidgetId]){
      cycleSubList[media.parentWidgetId] = {
        count: media.countplay,
        Israndom: media.IsRandom,
        counter: media.countplay,
        mediaIdx: -1,
        mediaId: []
      }
    }
    for (j = 0; j < cycleSubList[media.parentWidgetId].mediaId.length; j++) {
      if (cycleSubList[media.parentWidgetId].mediaId[j].mid === media.id) {
        mediaflag = 1;
        break;
      }
    }
    if (!mediaflag) {
      cycleSubList[media.parentWidgetId].mediaId.push({
        mid: media.id,
        media: media,
      })
    }
  }
  /**
   * Get the next widget from the cycle
   * @param {String} media The media we want to check for cycle playing.
   * @returns {Number}  how many times this media played .
   */
  this.counter = function (media) {
    var parentWid = cycleSubList[media.parentWidgetId];
    if (parentWid.count == parentWid.counter) {
      parentWid.counter = 0;
    } else {
      parentWid.counter += 1;
    }
    return parentWid.counter;
  }
  this.getNextWidgetFromCycle = function (media) {
    var c;
    var current_mediaIdx = cycleSubList[media.parentWidgetId].mediaIdx;
    var allMedia = cycleSubList[media.parentWidgetId].mediaId;

    c = CacheManager.counter(media);

    if (c === 0 ){
      CacheManager.counter(media);
      if (cycleSubList[media.parentWidgetId].Israndom == "1") {
        current_mediaIdx = Math.floor(Math.random() * allMedia.length);
      } else {
        current_mediaIdx += 1;
        if (current_mediaIdx >= allMedia.length) {
          current_mediaIdx = 0;
        }
      }
    }
    cycleSubList[media.parentWidgetId].mediaIdx = current_mediaIdx;
    media = allMedia[current_mediaIdx].media;
    return media;
  };

  /**
   * Add a play count for the specific layout.
   * @param {string} layoutId The id of the layout.
   * @param {string} scheduleid The schedule id of the layout.
   */
  this.mpph_addPlay = function (layoutId, scheduleid) {
      var layout = layoutId + '_' + scheduleid
      if (!maxPlaysPerHour[layout]) {
          CacheManager.mpph_getPlays(layoutId, scheduleid);
    }
      maxPlaysPerHour[layout].plays += 1;

  };
  /**
   * Get the number of plays in the current hour.
   * @param {string} layoutId The id of the layout.
   * @param {string} scheduleid The schedule id of the layout.
   * @returns {Object} An object with the plays and the time started/reset.
   */
  this.mpph_getPlays = function (layoutId, scheduleid) {
      var layout = layoutId + "_" + scheduleid
      if (!maxPlaysPerHour[layout]) {
          maxPlaysPerHour[layout] = {
              plays: 0,
              time: new Date()
      }
    }
      return maxPlaysPerHour[layout];
  };

  /**
   * Reset the plays per hour and the time.
   * @param {string} layoutId The id of the layout.
   * @param {string} scheduleid The schedule id of the layout.
   */
  this.mpph_reset = function (layoutId, scheduleid) {
      var layout = layoutId + '_' + scheduleid
      if (!maxPlaysPerHour[layout]) {
          CacheManager.mpph_getPlays(layoutId, scheduleid);
      } else {
          maxPlaysPerHour[layout].plays = 0;
          maxPlaysPerHour[layout].time = new Date();
    }
  };
    /**
     * Sets the layout into a campaign
     * @param groupkey
     * @param campaignLayout
     */
    this.setCampaignOrder = function (scheduleid, campaignLayout) {
        campaignOrder[scheduleid] = campaignLayout;
  }
    /**
     * Returns all the saved campaigns
     * @returns {string[]}
     */
    this.getCampaignOrder = function (scheduleid) {
        return campaignOrder[scheduleid];
  }
    /**
     * Returns all the saved campaigns
     * @returns {string[]}
     */
  this.getCampaignOrderKeys = function (){
    return Object.keys(campaignOrder);
  }
    /**
     * Delete a certain campaign
     * @param scheduleid
     */
    this.deleteCampaign = function (scheduleid) {
        delete campaignOrder[scheduleid];
  }
    /**
     * Sets the campaigns that is displaying right now
     * @param scheduleid
     */
    this.setCurrentGroupKey = function (scheduleid) {
        currentGroupKey = scheduleid;
  }
    /**
     * Gets the campaigns that is displaying right now
     * @returns {number}
     */
  this.getCurrentGroupKey = function (){
    return currentGroupKey;
  }
  // </editor-fold>
  // <editor-fold defaultstate="collapsed" desc="Native CMS Calls">
  
  // </editor-fold>
};